#include "RainInfo.h"
#include "RainLut.h"
#include "MyView.h"
#include "RainDrop.h"

void  _get_screen_bitmap_(BBitmap *,BRect,bool);

RainInfo::RainInfo( BView *view, int32 intensity )
						: mDrops(NULL),
							mIsRunning( true ),
							mView( NULL ),
							mHasDrawn( false ),
							mOldScreen( NULL ),
							mSaverThread( -1 ),
							mIntensity( intensity ),
							mNumDrops( 0 )
{
	for (int32 i = 0; i < kNumLuts; i++ ){
		mLuts[ i ] = NULL;
	}
	
	{
		BScreen screen( B_MAIN_SCREEN_ID ); //mScreenNum
		
		mScreenRect = screen.Frame();
		mColorSpace = screen.ColorSpace();
	}
		
	mOldScreen = new BBitmap( mScreenRect, mColorSpace, true );
	if ( !mOldScreen ){
		return;
	}

	_get_screen_bitmap_( mOldScreen, mScreenRect, false );

	view->Window()->Lock();
	mView = new MyView( &mHasDrawn, mOldScreen, view->Bounds(), "Pond", 0, B_WILL_DRAW );
	view->Window()->Unlock();
	
	int16 width = 1 + mScreenRect.right - mScreenRect.left;
	for (int32 i = 0; i < kNumLuts - 1; i++ ){
		mLuts[ i ] = new RainLut( i, false, width );
	}
	mLuts[ kNumLuts - 1 ] = new RainLut( kNumLuts - 1, true, width );
	
	mSaverThread = spawn_thread( RainInfo::SaverThreadStart,
										"Rain Maker",
										B_NORMAL_PRIORITY,
										(void*)(this) );
	
	if ( mSaverThread < B_NO_ERROR ){
		return;
	}
	
	return;
}

RainInfo::~RainInfo()
{
	for (int32 i = 0; i < kNumLuts; i++ ){
		delete mLuts[ i ];
	}

	BWindow *win = mView->Window();
	win->Lock();
	mView->Parent()->RemoveChild( mView );
	win->Unlock();
	
	delete mView;
	
	delete mOldScreen;
	
	return;
}

long RainInfo::StartThreads()
{
	int	i;
	
	resume_thread( mSaverThread );
	
	return B_NO_ERROR;
}

long RainInfo::SaverThreadStart( void *data )
{
	long	result;

	result = ( (RainInfo*)data )->RealStart();

	return result;
}

long RainInfo::RealStart( void )
{
	while ( mHasDrawn == false ){
		if ( !mIsRunning )
			return B_NO_ERROR;
		snooze( 1000 );
		// Sit 'n spin
	}

	while ( mIsRunning ){
		AddRandomDrop( mIntensity );
		DeleteDeadDrops();
		snooze( 1000 );
	}

	DeleteDrops();
	
	return B_NO_ERROR;
}

void RainInfo::AddRandomDrop( int16 intensity )
{
	int32	ranVal;
	int32	ranMod;
	int32	x;
	int32	y;
	
	if ( mNumDrops > kMaxDrops )
		return;
		
	ranVal = rand();
	ranMod = ranVal % ( 201 - intensity );
	
	if ( ranMod != 0 )
		return;
	
	mNumDrops++;
		
	ranVal = rand();
	ranMod = ranVal % (int32)( mScreenRect.right - mScreenRect.left );
	x = mScreenRect.left + ranMod;
		
	ranVal = rand();
	
	int32 height =  mScreenRect.bottom - mScreenRect.top - RainLut::kLutWidth - 2;
	ranMod = ranVal % height;
	y = mScreenRect.top + ranMod;
		
	RainDrop *drop = new RainDrop( x,
									y,
									RainLut::kLutWidth, 
									mColorSpace, 
									mOldScreen, 
									(BView*)mView,
									&(mLuts[0]),
									kNumLuts );
	
	drop->mNextDrop = mDrops;
	mDrops = drop;
	
	drop->StartThread();
	
	return;
}

void RainInfo::DeleteDeadDrops()
{
	long result;
		
	while ( mDrops ){
		if ( !mDrops->mAlive ){
			wait_for_thread( mDrops->mThread, &result );
			RainDrop *oldDrop;
			
			oldDrop = mDrops;
			mDrops = mDrops->mNextDrop;
			
			mNumDrops--;
					
			delete oldDrop;
		}else{
			
			RainDrop *aDrop = mDrops;
			
			while ( aDrop && aDrop->mNextDrop ){

				if ( !aDrop->mNextDrop->mAlive ){
					wait_for_thread( aDrop->mNextDrop->mThread, &result );
					RainDrop *oldDrop;
					
					oldDrop = aDrop->mNextDrop;
					aDrop->mNextDrop = aDrop->mNextDrop->mNextDrop;
					
					mNumDrops--;
					
					delete oldDrop;
				}else{
					aDrop = aDrop->mNextDrop;
				}

			}
			
			return;
		}		
	}
	return;
}

void RainInfo::DeleteDrops()
{
	RainDrop *drop;
	
	drop = mDrops;
	
	while ( drop ){
		drop->mAlive = false;
		drop = drop->mNextDrop;
	}
	
	DeleteDeadDrops();
	
	return;
}

long RainInfo::StopThreads()
{
	long exitValue;

	mIsRunning = false;

	wait_for_thread( mSaverThread, &exitValue );

	
	return B_NO_ERROR;
}


